********************************************************************************
* Conscious Coupling                                                           *
* Impact of a Skills Intervention to Address Intrahousehold					   *
* Constraints for Women Entrepreneurs in Ethiopia                              *
* Master Replication Do-File                                                   * 
* Author: Dumebi Ochem                                                         * 
* Date: 30 May 2025                                                            * 
* Last Reviewed: 17 Jun 2025                                                   * 
* Version: 12.1                                                                * 
********************************************************************************

/*
* Datasets Required:
*  - screening: 210328_WB_Couples_Training_Phone_Screening_Final_public,
				210728_WB_Couples_Training_Phone_Screening_Final_public,
				211206_WB_Couples_Training_Phone_Screening_round2_public,
				220530_WB_Couples_Training_Phone_Screening_round3_public
*  - baseline:  210927_WB_Couples_Training_BL_Clean_public,
				220204_WB_Couples_Training_BL_Clean_r2_public,
				DOT_Couples_Training_strat_randomization,
				DOT_Couples_Training_strat_randomization_R2BL,
				230811_WBC_initialduplicatescheck, DOT_duplicates_check,
				WB_Couples_Training_Implementation_Monitoring
*  - endline:   WB_Couples_Training_EL_main_public, WB_Couples_Training_EL_spouse_public
*
* Unique Identificators: dot_1_1 (screening data) meta_1_1 (all other data)
*
* Outline: 
*  1. Preparation and Main Settings
*      - Installs required packages (if needed)
*      - Defines global paths and standard settings for reproducibility
*      - Specifies sections of the analysis to be executed
*
*  2. Data Cleaning and Construction
*      - Executes dofiles responsible for cleaning and preparing datasets
*
*  3. Data Analysis
*      - Runs scripts that generate descriptive statistics
*      - Produces regression tables
*/


* ------------------------------------------------------------------------------ 
* Set Default Options and Memory Limits
* ------------------------------------------------------------------------------ 
clear 			all
macro drop 		_all
set more 		off
set varabbrev 	off
cap log 		close

version 		12.1

set maxvar 		32767
set matsize 	11000
set min_memory	0
set max_memory	.
set segmentsize	32m
set niceness	5	


* ------------------------------------------------------------------------------ 
* Define Globals Directory
* ------------------------------------------------------------------------------ 
/* 
Note: this code assumes that the underlying folder structure of this replication
package has remained unchanged. If you have only downloaded some of the do-files, 
or changed the folder structure, you will need to change your globals before 
continuing.
*/

gl rep_path		"your path/Reproducibility package"
gl dofiles		"$rep_path/Do files"
gl data_final	"$rep_path/Data"
gl data_raw		"$rep_path/Data/Raw"
gl data_working "$rep_path/Data/Intermediate"
gl tables_main  "$rep_path/Outputs/Main Tables"
gl tables_app   "$rep_path/Outputs/Appendix Tables"


* ------------------------------------------------------------------------------ 
* Install Required Packages (Change to 1 to install on first-time setup)
* ------------------------------------------------------------------------------ 
sysdir set PLUS "$rep_path/Ado Files"

if 0{
	ssc install ietoolkit, replace
	ssc install outreg2,   replace
	ssc install estout,    replace
	ssc install coefplot,  replace
	ssc install swindex,   replace
	ssc install winsor,    replace
	ssc install qregplot,  replace
	ssc install ftools,	   replace
	ssc install split,     replace
	ssc install elabel,    replace
}


* ------------------------------------------------------------------------------ 
*  Define Sections to Execute
* ------------------------------------------------------------------------------ 
	* Set equal to 1 if needed
	local CleaningConstruction 1
	local Analysis 			   1


* ------------------------------------------------------------------------------ 
*  Data Cleaning and Construction
* ------------------------------------------------------------------------------ 
if (`CleaningConstruction' == 1) {
	* Initial cleaning of all datasets
    do "$dofiles/1_DuplicatesCleaningDofile.do"
	
	* Construction of screening and baseline datasets
    do "$dofiles/2_BaselineConstructionDofile.do"
	
	* Construction of endline datasets
    do "$dofiles/3_EndlineConstructionDofile.do"
	
	* Assemble final dataset for analysis
    do "$dofiles/4_FinalCleaningDofile.do"
}


* ------------------------------------------------------------------------------ 
*  Analysis Tables and Figures
* ------------------------------------------------------------------------------ 
if (`Analysis' == 1) {
	* Randomization and take-up balance tables
    do "$dofiles/5_DescriptivesDofile.do"
	
	* Main regression tables
    do "$dofiles/6_MainAnalysisDofile.do"
	
	* Appendix regression tables
    do "$dofiles/7_AppendixAnalysisDofile.do"
}  

********************************************************************************
*                            END OF MASTER DOFILE                              *
********************************************************************************
