********************************************************************************
* Conscious Coupling                                                           *
* Cleaning and Construction: Endline Datasets                                  *
********************************************************************************


**#  1. Merging Main and Spouse Datasets
* -----------------------------------------------------------------------------*
	use 	     "$data_raw/WB_Couples_Training_EL_spouse_public.dta", clear // no duplicates

	keep 		 meta_1_1 resp_id survey_status final_status_spouse bus_sector ///
				 bus_register bus_empl resp_age spouse_city_pulled meta_1_8-fin_1_16_5 today
	qui ds		 meta_1_1 resp_id final_status_spouse, not
	foreach i of varlist `r(varlist)' {
				 rename `i' sp_`i'
				 label drop _all
				 }
*	elabel ren   (*) (*_sp)
	replace 	 final_status_spouse = "NOT TRACKED" if final_status_spouse == ""
	
	tempfile  	 spouse_data
	save     	 `spouse_data'

	* Merge to main dataset
	use 	     "$data_raw/WB_Couples_Training_EL_main_public.dta", clear // no duplicates
	
	replace 	 a_bus_1_10_1 = 1 if a_bus_1_10_1 == -1 // fixed typo
	keep 		 meta_1_1 resp_id final_status_main bus_sector bus_register bus_empl ///
				 resp_age resp_age_pulled meta_1_8-still_together has_partner-sjt38_time ///
				 a_bus_1_4_2_oth_1 a_bus_1_new5_1_oth sjt15_6_oth sjt15_8_oth ///
				 main_business_sector today
	merge 		 1:1 meta_1_1 resp_id using `spouse_data', nogen

	* Merge treatment status
	rename 		 (meta_1_1 resp_id) (temp meta_1_1) 		  
	merge 		 1:1 meta_1_1 using "$data_working/DOT_treatment_status.dta", ///
				 keepusing(treatment_new strata_new) keep(1 3) nogen
	rename 		 (meta_1_1 temp) (resp_id meta_1_1)


**#  2. Creating Main Outcome Variables
* -----------------------------------------------------------------------------*
	gen 		 control = (treatment_new == 0) // control group for computing Anderson indices

	
	*** A. Business outcomes and practices
	destring 	 main_business_employees_final, replace force
	destring 	 main_business_employees, 		replace force

	foreach i in b_bus_1_25 b_bus_1_26 b_bus_1_27 b_bus_1_28 b_bus_1_39 a_bus_1_9_1 a_bus_1_9_2 ///
				 main_business_employees main_business_employees_final b_bus_1_31 b_bus_1_33 ///
				 b_bus_1_35 b_bus_1_37 b_bus_1_50 b_bus_1_41 meta_1_22 meta_1_25 intra_1_33 ///
				 fin_1_10a fin_1_10b fin_1_10c fin_1_10d fin_1_new1 sp_h_labor_1_16_1 ///
				 sp_h_labor_1_17_1 sp_h_labor_1_18_1 sp_h_labor_1_20_1 sp_meta_1_20 sp_meta_1_29 {
				 label values `i' .
				 recode `i' (-99 -98 -97 -96 -95 -94 .=0)
				 } 

	* Convert to real values
	/* Inflation rate (CPI base: December 2016)
		- Jun 2023: 337.6
		- Jul 2023: 344.8
		- Aug 2023: 352.1
		- Sep 2023: 359.6 */
	gen double   date  = date(today, "DMY")
	format 		 date %td
	gen 		 month = month(date)

	foreach i in b_bus_1_25 b_bus_1_26 b_bus_1_27 b_bus_1_28 b_bus_1_31 b_bus_1_33 b_bus_1_35 ///
				 b_bus_1_37 b_bus_1_41 meta_1_22 meta_1_25 intra_1_33 fin_1_10a fin_1_10b fin_1_10c ///
				 fin_1_10d fin_1_new1 {
				 replace `i' = `i'/(3.376) if month == 6
				 replace `i' = `i'/(3.448) if month == 7
				 replace `i' = `i'/(3.521) if month == 8
				 replace `i' = `i'/(3.596) if month == 9
				 }

	egen 		 machinery = rowtotal(b_bus_1_31 b_bus_1_33)
	clonevar 	 inventory = b_bus_1_35
	clonevar 	 raw = b_bus_1_37
	egen 		 capital = rowtotal(b_bus_1_31 b_bus_1_33 b_bus_1_35 b_bus_1_37)

	gen 		 employees = main_business_employees
	replace 	 employees = main_business_employees_final if b_bus_1_15 == 0
	replace 	 employees = 0 if employees == .

	* Winsorinzing and transformations
	foreach i in b_bus_1_25 b_bus_1_26 b_bus_1_27 b_bus_1_28 b_bus_1_39 employees ///
				 capital machinery inventory raw {
				 winsor `i', gen (win99_`i') p(0.01)
				 winsor `i', gen (win95_`i') p(0.05)
				 winsor `i', gen (win90_`i') p(0.10)
				 winsor `i', gen (win85_`i') p(0.15)
				 winsor `i', gen (win80_`i') p(0.20)
				 winsor `i', gen (win75_`i') p(0.25)
				 gen ihs_`i' = asinh(`i')
				 gen log_`i' = log(`i')
				 }
	gen 		 any_profits = (b_bus_1_28 > 0)
	gen 		 win95_any_profits = (win95_b_bus_1_28 > 0)

	* Business survival
	gen 		 D_survival = (inlist(a_bus_1_new0,1,2)) // open or temporarily closed
	replace 	 D_survival = 0 if a_bus_1_new7 == 3 // answered no longer operating a business
	replace 	 D_survival = 0 if a_bus_1_new7 == 2 & b_bus_1_new1 == 1
	// original business closed and operating new business
	replace 	 D_survival = 0 if a_bus_1_new7 == 2 & a_bus_1_new0 == 2
	// original business temporarily closed and operating new business
	gen D_exit = (a_bus_1_new3 == 1) // exit from entrepreneurship

	label define activity 0 "Not working" /*includes other*/ 1 "Original business" ///
						  2 "New business in same sector" 3 "New business in different sector" ///
						  4 "Wage employment", replace
	gen 		 activity = 0 if inlist(a_bus_1_new5_p,0,8,9)
	replace 	 activity = 0 if inlist(a_bus_1_new5_1,8,9)
	replace 	 activity = 0 if inlist(a_bus_1_new5_1_oth,"Staying At Home", ///
				 "Been Bet Kiray Gebi Ena Betureta")
	replace 	 activity = 0 if a_bus_1_new5_p == 1 & a_bus_1_new5_1 == 1 & D_survival == 0 // 10
	replace 	 activity = 1 if a_bus_1_new5_p == 1 & a_bus_1_new5_1 == 1 & D_survival == 1 // 700
	replace 	 activity = 1 if inlist(a_bus_1_new5_p,.,0) & D_survival == 1
	replace 	 activity = 1 if a_bus_1_new5_1 == 1 & a_bus_1_new7 == 1 & D_survival == 1
	replace 	 activity = 1 if a_bus_1_new5_p == 1 & a_bus_1_new7 == 1 & D_survival == 1
	replace 	 activity = 2 if a_bus_1_new5_p == 1 & D_survival == 0 & a_bus_1_new5_1 == 4
	replace 	 activity = 2 if D_survival == 1 & a_bus_1_new7 == 2 & a_bus_1_new5_1 == 4
	replace 	 activity = 2 if D_survival == 1 & a_bus_1_new7 == 1 & a_bus_1_new5_1 == 4 ///
				 & activity == .
	replace 	 activity = 3 if a_bus_1_new5_p == 1 & D_survival == 0 & a_bus_1_new5_1 == 5
	replace 	 activity = 3 if a_bus_1_new5_p == 2 & D_survival == 0 & a_bus_1_new5_1 == 5
	replace 	 activity = 3 if D_survival == 1 & a_bus_1_new7 == 2 & a_bus_1_new5_1 == 5
	replace 	 activity = 3 if D_survival == 1 & a_bus_1_new7 == 1 & a_bus_1_new5_1 == 5 ///
				 & activity == .
	replace 	 activity = 4 if inlist(a_bus_1_new5_p,2,3,10) & inlist(a_bus_1_new5_1,2,3,10)
	replace 	 activity = 4 if a_bus_1_new5_p == 1 & D_survival == 0 & inlist(a_bus_1_new5_1,2,3,10)
	replace 	 activity = 4 if a_bus_1_new5_1 == 1 & a_bus_1_new7 == 2 & inlist(a_bus_1_new5_p,2,3,10)
	replace 	 activity = 4 if a_bus_1_new5_1_oth == "Ngo"
	replace 	 activity = 0 if a_bus_1_new5_p == 1 & D_survival == 0 & inlist(a_bus_1_new5_1,6,7)
	replace 	 activity = 0 if activity == .
	label values activity activity

	gen 		 D_newsamesector = (activity == 2)
	gen 		 D_newdiffsector = (activity == 3)

	* Business practices
	label define satisfied 1 "Not at all satisfied" 2 "Not very satisfied" ///
						   3 "Fairly satisfifed" 4 "Very satisfied", replace
	clonevar 	 qualbusiness = char_1_68
	replace 	 qualbusiness = 5 - qualbusiness if !inlist(char_1_68,0,.)
	label values qualbusiness satisfied
						   
	clonevar 	 D_newbus = b_bus_1_50
	gen 		 D_aspiration = (b_bus_1_47 == 1)
	gen 		 D_innovation = (b_bus_1_48 == 1 | b_bus_1_49 == 1)
	
	gen 		 D_busplan = (b_bus_1_22 == 1)
	gen 		 D_budget = (inlist(b_bus_1_23,1,2))
	gen 		 D_records = (b_bus_1_24 != 6)
	egen 		 recordkeep = rowtotal(D_busplan D_budget D_records) // record-keeping

	gen 		 D_bus_45_1 = (b_bus_1_45_1 > 0 & b_bus_1_45_1 < .)
	gen 		 D_bus_45_2 = (b_bus_1_45_2 > 0 & b_bus_1_45_2 < .)
	gen 		 D_bus_45_6 = (b_bus_1_45_6 > 0 & b_bus_1_45_6 < .)
	gen 		 D_bus_45_11 = (b_bus_1_45_11 > 0 & b_bus_1_45_11 < .)
	gen 		 D_bus_45_12 = (b_bus_1_45_12 > 0 & b_bus_1_45_12 < .)
	gen 		 D_bus_45_13 = (b_bus_1_45_13 > 0 & b_bus_1_45_13 < .)
	egen 		 marketing = rowtotal(D_bus_45_1 D_bus_45_2 D_bus_45_6 D_bus_45_11 ///
				 D_bus_45_12 D_bus_45_13) // marketing

	gen 		 D_bus_45_3 = (b_bus_1_45_3 > 0 & b_bus_1_45_3 < .)
	gen 		 D_bus_45_9 = (b_bus_1_45_9 > 0 & b_bus_1_45_9 < .)
	gen 		 D_bus_45_10 = (b_bus_1_45_10 > 0 & b_bus_1_45_10 < .)
	egen 		 stockcontrol = rowtotal(D_bus_45_3 D_bus_45_9 D_bus_45_10) // stock control

	gen 		 D_bus_45_4 = (b_bus_1_45_4 > 0 & b_bus_1_45_4 < .)
	gen 		 D_bus_45_5 = (b_bus_1_45_5 > 0 & b_bus_1_45_5 < .)
	gen 		 D_bus_45_7 = (b_bus_1_45_7 > 0 & b_bus_1_45_7 < .)
	gen 		 D_bus_45_8 = (b_bus_1_45_8 > 0 & b_bus_1_45_8 < .)
	egen 		 finplanning = rowtotal(D_bus_45_4 D_bus_45_5 D_bus_45_7 ///
				 D_bus_45_8) // financial planning


	*** B. Socio-emotional skills (NB: entrepreneural identity not asked at endline)
	foreach i in char_1_new19 char_1_new21 char_1_new24 char_1_new25 char_1_new26 {
				 replace `i' = 6 - `i' if !inlist(`i',0,.) // reverse values so that 5 means high
				 }

	egen 		 selfesteem = rmean(char_1_new17-char_1_new26) // self-esteem
	egen 		 selfefficacy = rmean(char_1_2-char_1_11) // self-efficacy
	egen 		 persgrowth = rmean(char_1_12-char_1_15) // personal growth
	egen 		 proactivity = rmean(char_1_25-char_1_32) // proactivity

	egen 		 cognempathy = rmean(intra_1_new5-intra_1_new9) // cognitive empathy
	foreach i in sjt38_1-sjt38_6 {
				 recode `i' (-99 -98 -97 -96 -95 -94 =.)
				 }
	egen 		 cognempathy_sjt = rmean(sjt38_1-sjt38_6)

	egen 		 express = rmean(char_1_new12-char_1_new16) // expressiveness
	foreach i in sjt26_1-sjt26_6 {
				 recode `i' (-99 -98 -97 -96 -95 -94 =.)
				 }
	egen 		 express_sjt = rmean(sjt26_1-sjt26_6)

	foreach i in sjt15_1-sjt15_4 sjt15_7 {
				 recode `i' (-99 -98 -97 -96 -95 -94 =.)
				 }
	replace 	 sjt15_7 = 6 - sjt15_7 if !inlist(sjt15_7,0,.)
	egen 		 influence = rmean(sjt15_1-sjt15_4 sjt15_7) // interpersonal influence

	foreach i in sjt17_1-sjt17_9 {
				 recode `i' (-99 -98 -97 -96 -95 -94 99/*typo*/ =.)
				 }
	replace 	 sjt17_1 = 6 - sjt17_1 if !inlist(sjt17_1,0,.)
	replace 	 sjt17_7 = 6 - sjt17_7 if !inlist(sjt17_7,0,.)
	egen 		 negotiate = rmean(sjt17_1-sjt17_9) // negotiation

	* Standardized Z-scores
	foreach v in qualbusiness selfesteem selfefficacy persgrowth proactivity cognempathy ///
				 cognempathy_sjt express express_sjt influence negotiate ///
				 recordkeep marketing stockcontrol finplanning {
				 qui sum `v'
				 gen `v'_z = ((`v' - r(mean))/r(sd))
				 }

	* Composite Anderson indices
	swindex 	 D_busplan D_budget D_records D_bus_45_1 D_bus_45_2 D_bus_45_6 D_bus_45_11 ///
				 D_bus_45_12 D_bus_45_13 D_bus_45_3 D_bus_45_9 D_bus_45_10 D_bus_45_4 /// 
				 D_bus_45_7 D_bus_45_8, g(IND_buspractice) normby(control)
	swindex 	 char_1_new17-char_1_new26 char_1_2-char_1_11 char_1_12-char_1_15 /// 
				 char_1_25-char_1_32, g(IND_ses1) normby(control) // intrapersonal skills
	swindex 	 sjt38_1-sjt38_6 sjt26_1-sjt26_6 sjt15_1-sjt15_4 sjt15_7 sjt17_1-sjt17_9, ///
				 g(IND_ses2) normby(control) // interpersonal skills
	swindex 	 char_1_new17-char_1_new26 char_1_2-char_1_11 char_1_12-char_1_15 ///
				 char_1_25-char_1_32 sjt38_1-sjt38_6 sjt26_1-sjt26_6 sjt15_1-sjt15_4 ///
				 sjt15_7 sjt17_1-sjt17_9, g(IND_ses_all) normby(control)
	swindex 	 char_1_new17-char_1_new26 char_1_2-char_1_11 char_1_12-char_1_15 ///
				 char_1_25-char_1_32 intra_1_new5-intra_1_new9 char_1_new12-char_1_new16 ///
				 sjt38_1-sjt38_6 sjt26_1-sjt26_6 sjt15_1-sjt15_4 sjt15_7 sjt17_1-sjt17_9, ///
				 g(IND_ses_all_v2) normby(control)
				 
		
	*** C. Marriage quality	
	egen 		 coop = rowtotal(intra_1_22_1 intra_1_22_2 intra_1_22_3 intra_1_22_4), missing // cooperation
	egen 		 empathy = rowmean(char_1_46-char_1_50) // empathy
	replace 	 empathy = . if meta_1_13 != 2
	egen 		 commun = rowmean(char_1_52 char_1_53) // communication
	replace 	 commun = . if meta_1_13 != 2
	
	replace 	 char_1_new3 = 6 - char_1_new3
	egen 		 trust = rowmean(char_1_new1 char_1_new2 char_1_new3) // trust
	replace 	 trust = . if meta_1_13 != 2
	
	egen 		 abuse = rowtotal(intra_1_new20a-intra_1_new20e), missing // controlling spouse
	replace 	 abuse = . if meta_1_13 != 2
	gen 		 D_abuse = (abuse > 0 & abuse < .)
	replace 	 D_abuse = . if meta_1_13 != 2
	
	clonevar 	 intimacy = intra_1_49 // intimate relationship
	replace 	 intimacy = 6 - intimacy if !inlist(intra_1_49,0,.)
	label define intimate 1 "Hostile" 2 "Apathetic" 3 "Loyal" 4 "Congenial" 5 "Intimate", replace
	label values intimacy intimate
	
	clonevar 	 qualmarriage = char_1_16 // marriage satisfaction
	clonevar 	 quallife = char_1_67 // life satisfaction
	replace 	 quallife = 5 - quallife if !inlist(char_1_67,0,.)
	label values quallife satisfied
	
	* Standardized Z-scores
	foreach v in coop empathy commun trust abuse intimacy qualmarriage quallife {
				 qui sum `v'
				 gen `v'_z = ((`v' - r(mean))/r(sd))
				 }
				 
	* Anderson index
	swindex 	 intra_1_22_1 intra_1_22_2 intra_1_22_3 intra_1_22_4 char_1_46-char_1_50 /// 
				 char_1_52 char_1_53 char_1_new1 char_1_new2 char_1_new3, ///
				 g(IND_marriage) normby(control)
	replace 	 IND_marriage = . if meta_1_13 != 2
	qui sum 	 IND_marriage if final_status_main == "COMPLETED", d
	gen 		 D_marriage = (IND_marriage > r(p50))
	replace 	 D_marriage = . if meta_1_13 != 2			 


	*** D. Household outcomes
	gen 		 inc_share = (meta_1_22/meta_1_25)*100
	replace 	 inc_share = 0 if inc_share == .
	gen 		 D_mainincearn = (meta_1_24 == 1)
	gen 		 D_breadwinner = (meta_1_new0 == 1)

	* Transformations
	rename (meta_1_24 meta_1_25) (meta_1_24_2 meta_1_24)
	foreach i in meta_1_22 meta_1_24 {
				 winsor `i', gen (win99_`i') p(0.01)
				 winsor `i', gen (win95_`i') p(0.05)
				 winsor `i', gen (win90_`i') p(0.10)
				 winsor `i', gen (win85_`i') p(0.15)
				 winsor `i', gen (win80_`i') p(0.20)
				 winsor `i', gen (win75_`i') p(0.25)
				 gen ihs_`i' = asinh(`i')
				 gen log_`i' = log(`i')
				 }


	*** E. Financial outcomes						
	clonevar 	 D_loan_all = fin_1_2
	clonevar 	 D_loan = fin_1_11
	clonevar 	 loan_size = fin_1_10a
	gen 		 D_loan_now = (loan_size > 0 & loan_size != .)
	gen 		 D_altfinance = strpos(fin_1_1,"2") > 0 | strpos(fin_1_1,"3") > 0
	egen 		 altfinance_size = rowtotal(fin_1_10b-fin_1_10d)
	replace 	 altfinance_size = 0 if D_altfinance == 0
	clonevar 	 savings = fin_1_new1 // savings

	* Winsorizing
	foreach i in loan_size altfinance_size savings {
				 cap winsor `i', gen (win99_`i') p(0.01)
				 cap winsor `i', gen (win95_`i') p(0.05)
				 cap winsor `i', gen (win90_`i') p(0.10)
				 cap winsor `i', gen (win85_`i') p(0.15)
				 cap winsor `i', gen (win80_`i') p(0.20)
				 cap winsor `i', gen (win75_`i') p(0.25)
				 gen ihs_`i' = asinh(`i')
				 gen log_`i' = log(`i')
				 }


	*** F. Decision-making
	label define decision_making 1 "Self" 2 "Spouse" 3 "Joint HH" 4 "Other HH" ///
								 5 "Joint non-HH" 6 "Other", replace
								 
	foreach i of numlist 4 9 11 12 13 {
		gen 	  decision`i' = . if inlist(intra_1_`i'a,""," ","-97") // missing for non-married women
		replace   decision`i' = 1 if intra_1_`i'a=="1"
		replace   decision`i' = 2 if intra_1_`i'a=="2"
		replace   decision`i' = 3 if ///
		(intra_1_`i'a=="1 2"|intra_1_`i'a=="1 2 3"|intra_1_`i'a=="1 3")
		replace   decision`i' = 4 if (intra_1_`i'a=="3"|intra_1_`i'a=="2 3")
		replace   decision`i' = 5 if ///
		(intra_1_`i'a=="1 4"|intra_1_`i'a=="1 2 4"|intra_1_`i'a=="1 3 4"|intra_1_`i'a=="1 2 3 4")
		replace   decision`i' = 6 if (intra_1_`i'a=="4"|intra_1_`i'a=="2 4"|intra_1_`i'a=="3 4")
		replace   decision`i' = . if meta_1_13 != 2
		label val decision`i' decision_making
	 
		gen 	  D_self_decision`i'  = (inlist(decision`i',1))
		replace   D_self_decision`i'  = . if decision`i' == .
		gen 	  D_husb_decision`i'  = (inlist(decision`i',2))
		replace   D_husb_decision`i'  = . if decision`i' == .
		gen 	  D_joint_decision`i' = (inlist(decision`i',3))
		replace   D_joint_decision`i' = . if decision`i' == .
		}
		
	gen 		 D_autonomy = (b_bus_1_14 == 1)
	gen 		 D_decisions = (intra_1_17 == 1) // decisions on profits

	* Anderson indices
	swindex 	 D_self_decision9 D_self_decision13, g(IND_self_bus_decision) normby(control)
	swindex 	 D_self_decision4 D_self_decision11 D_self_decision12, ///
				 g(IND_self_hh_decision) normby(control)
	swindex 	 D_husb_decision9 D_husb_decision13, g(IND_husb_bus_decision) normby(control)
	swindex 	 D_husb_decision4 D_husb_decision11 D_husb_decision12, ///
				 g(IND_husb_hh_decision) normby(control)
	swindex 	 D_joint_decision9 D_joint_decision13, g(IND_joint_bus_decision) normby(control)
	swindex 	 D_joint_decision4 D_joint_decision11 D_joint_decision12, ///
				 g(IND_joint_hh_decision) normby(control)
	
	* Simple indices
	egen 		 D_self_bus_decision = anymatch(D_self_decision9 D_self_decision13), v(1)
	replace 	 D_self_bus_decision = . if meta_1_13 != 2
	egen 		 temp = rmean(D_self_decision9 D_self_decision13)
	gen 		 D_self_bus_decision_v2 = (temp == 1)
	replace 	 D_self_bus_decision_v2 = . if meta_1_13 != 2
	drop 		 temp

	egen 		 D_self_hh_decision = anymatch(D_self_decision4 D_self_decision11 ///
				 D_self_decision12), v(1)
	replace 	 D_self_hh_decision = . if meta_1_13 != 2
	egen 		 temp = rmean(D_self_decision4 D_self_decision11 D_self_decision12)
	gen 		 D_self_hh_decision_v2 = (temp == 1)
	replace 	 D_self_hh_decision_v2 = . if meta_1_13 != 2
	drop 		 temp

	egen 		 D_husb_bus_decision = anymatch(D_husb_decision9 D_husb_decision13), v(1)
	replace 	 D_husb_bus_decision = . if meta_1_13 != 2
	egen 		 temp = rmean(D_husb_decision9 D_husb_decision13)
	gen 		 D_husb_bus_decision_v2 = (temp == 1)
	replace 	 D_husb_bus_decision_v2 = . if meta_1_13 != 2
	drop 		 temp

	egen 		 D_husb_hh_decision = anymatch(D_husb_decision4 D_husb_decision11 ///
				 D_husb_decision12), v(1)
	replace 	 D_husb_hh_decision = . if meta_1_13 != 2
	egen 		 temp = rmean(D_husb_decision4 D_husb_decision11 D_husb_decision12)
	gen 		 D_husb_hh_decision_v2 = (temp == 1)
	replace 	 D_husb_hh_decision_v2 = . if meta_1_13 != 2
	drop 		 temp

	egen 		 D_joint_bus_decision = anymatch(D_joint_decision9 D_joint_decision13), v(1)
	replace 	 D_joint_bus_decision = . if meta_1_13 != 2
	egen 		 temp = rmean(D_joint_decision9 D_joint_decision13)
	gen 		 D_joint_bus_decision_v2 = (temp == 1)
	replace 	 D_joint_bus_decision_v2 = . if meta_1_13 != 2
	drop 		 temp

	egen 		 D_joint_hh_decision = anymatch(D_joint_decision4 D_joint_decision11 ///
				 D_joint_decision12), v(1)
	replace 	 D_joint_hh_decision = . if meta_1_13 != 2
	egen 		 temp = rmean(D_joint_decision4 D_joint_decision11 D_joint_decision12)
	gen 		 D_joint_hh_decision_v2 = (temp == 1)
	replace 	 D_joint_hh_decision_v2 = . if meta_1_13 != 2
	drop 		 temp

	gen 		 D_interfere = (intra_1_new3 == 1) // spouse interference in business
	replace 	 D_interfere = . if meta_1_13 != 2

	clonevar 	 disagree = intra_1_new4 // total disagreements
	recode 		 disagree (-99 -98 -97 -96 -95 -94 =0)
	replace 	 disagree = . if meta_1_13 != 2
	gen 		 D_disagree = (disagree>0 & disagree!=.)
	replace 	 D_disagree = . if meta_1_13 != 2

	gen 		 D_hidearn = (meta_1_new6 == 0) // hides earnings from spouse
	replace 	 D_hidearn = . if meta_1_13 != 2


	*** G. Intra-household resource sharing
	foreach i in meta_1_28 meta_1_29 meta_1_30 meta_1_31 intra_1_29 intra_1_32 intra_1_33 {
				 recode `i' (-99 -98 -97 -96 -95 -94 . =0)
				 }

	clonevar 	 time_work_resp = meta_1_28 // time use
	replace  	 time_work_resp = . if meta_1_13 != 2
	clonevar 	 time_work_spouse = meta_1_30
	replace  	 time_work_spouse = . if meta_1_13 != 2
	clonevar 	 time_chores_resp = meta_1_29
	replace  	 time_chores_resp = . if meta_1_13 != 2
	clonevar 	 time_chores_spouse = meta_1_31
	replace  	 time_chores_spouse = . if meta_1_13 != 2

	egen 		 time = rowtotal(time_chores_resp time_chores_spouse), missing
	gen 		 share_chores = time_chores_resp/time
	replace 	 share_chores = 0 if time == 0
	gen 		 share_chores_sp = time_chores_spouse/time
	replace 	 share_chores_sp = 0 if time == 0

	gen 		 D_assetjoint = (intra_1_16 == 3) // asset ownership
	replace 	 D_assetjoint = . if meta_1_13 != 2

	gen 		 D_direct_help = (intra_1_25 == 1 | intra_1_26 == 1) // direct business help
	replace 	 D_direct_help = . if meta_1_13 != 2
	clonevar 	 direct_help = intra_1_29
	replace 	 direct_help = . if meta_1_13 != 2

	gen 		 D_foodprep = (!inlist(intra_1_21a,1,8,9)) // household help
	gen 		 D_clean = (!inlist(intra_1_21b,1,8,9))
	gen 		 D_chcare = (!inlist(intra_1_21c,1,8,9))
	replace 	 D_foodprep = . if meta_1_13 != 2
	replace 	 D_clean = .  if meta_1_13 != 2
	replace 	 D_chcare = . if meta_1_13 != 2 | intra_1_21c == 9

	gen 		 D_foodprep_2 = (inlist(intra_1_21a,1,2)) // this accounts for outside domestic help
	gen 		 D_clean_2 = (inlist(intra_1_21b,1,2))
	gen 		 D_chcare_2 = (inlist(intra_1_21c,1,2))
	replace 	 D_foodprep_2 = . if meta_1_13 != 2
	replace 	 D_clean_2 = .  if meta_1_13 != 2
	replace 	 D_chcare_2 = . if meta_1_13 != 2 | intra_1_21c == 9

	gen 		 D_support_house = (!inlist(intra_1_21a,1,8,9) | !inlist(intra_1_21b,1,8,9) | ///
				 !inlist(intra_1_21c,1,8,9))
	replace 	 D_support_house = . if meta_1_13 != 2
	gen 		 D_support_house_2 = (intra_1_21a == 1 & intra_1_21b == 1 & intra_1_21c == 1)
	replace 	 D_support_house_2 = . if meta_1_13 != 2
	gen 		 D_support_house_3 = (intra_1_21a == 1 | intra_1_21b == 1 | intra_1_21c == 1)
	replace 	 D_support_house_3 = . if meta_1_13 != 2
	gen 		 D_support_house_4 = (inlist(intra_1_21a,1,2) | inlist(intra_1_21b,1,2) | ///
				 inlist(intra_1_21c,1,2))
	replace 	 D_support_house_4 = . if meta_1_13 != 2

	gen 		 D_knowledge = (intra_1_36 == 1) // shared knowledge
	replace 	 D_knowledge = . if meta_1_13 != 2

	gen 		 D_networks = (intra_1_38 == 1) // shared networks
	replace 	 D_networks = . if meta_1_13 != 2

	egen 		 support_emotion = rmean(intra_1_50-intra_1_53) // emotional support
	replace 	 support_emotion = . if meta_1_13 != 2
	qui sum 	 support_emotion
	gen 		 support_emotion_z = ((support_emotion - r(mean))/r(sd))

	gen 		 D_finance = (intra_1_30 == 1 | intra_1_31 == 1) // financial support
	replace 	 D_finance = . if meta_1_13 != 2
	egen 		 support_finance = rowtotal(intra_1_32 intra_1_33)
	replace 	 support_finance = . if meta_1_13 != 2

	* Overall impact (not including help in the household)
	egen 		 D_support_any = anymatch(D_direct_help D_knowledge D_networks D_finance), v(1)
	replace 	 D_support_any = . if meta_1_13 != 2
	
	egen 		 benefit_spouse = rmean(intra_1_46a-intra_1_46f)
	replace 	 benefit_spouse = . if meta_1_13 != 2
	qui sum 	 benefit_spouse
	gen 		 benefit_spouse_z = ((benefit_spouse - r(mean))/r(sd))
	
	* Anderson index
	swindex 	 D_direct_help D_knowledge D_support_house D_networks D_finance /// 
				 intra_1_new1_1-intra_1_new1_10 intra_1_50-intra_1_53, ///
				 g(IND_support) normby(control)
	replace 	 IND_support = . if meta_1_13 != 2
	swindex 	 D_direct_help D_support_house D_knowledge D_networks D_finance ///
				 intra_1_50-intra_1_53, g(IND_support_old) normby(control)
	replace 	 IND_support_old = . if meta_1_13 != 2
	swindex 	 intra_1_new1_1-intra_1_new1_10, g(IND_support_new) normby(control)
	replace 	 IND_support_new = . if meta_1_13 != 2
	

**#  3. Creating Control Variables
* -----------------------------------------------------------------------------*
	gen 		 D_ScaleUp = (meta_1_new8 == 1)
	gen		 	 D_ScaleUp_1 = (meta_1_new9_1 == 1)
	gen 		 D_ScaleUp_2 = (meta_1_new9_2 == 1)
	gen 		 D_ScaleUp_3 = (meta_1_new9_3 == 1)
	gen 		 D_ScaleUp_4 = (meta_1_new9_4 == 1)
	gen 		 D_treated = (inlist(meta_1_new11,1,2))
	gen 		 D_treated_T1 = (inlist(meta_1_new11,1))
	gen 		 D_treated_T2 = (inlist(meta_1_new11,2))
	gen 		 flag_treated = (D_treated == 1 & meta_1_new10_0 == 1) // some contradictions

	
	*** A. Demographics
	gen 		 D_hhead = (meta_1_14 == 1)
	gen 		 D_married = (meta_1_13 == 2)
	clonevar 	 years_married = intra_1_44
	recode 		 years_married (-99 -98 -97 -96 -95 -94 .=0)

	clonevar 	 age = meta_1_10
	clonevar 	 sons = meta_1_16a
	clonevar 	 daughters = meta_1_16b
	clonevar 	 children = children_sum
	gen 		 hhsize = children + 2
	replace 	 hhsize = children + 1 if D_married == 0
	gen 		 D_children = (children > 0 & children < .)


	*** B. Education
	label define education 0 "No education" 1 "Incomplete primary education" 2 "Primary education" ///
						   3 "High school" 4 "Vocational education" 5 "Other diploma" ///
						   6 "Bachelors degree" 7 "Masters degree" 8 "PhD degree", replace
	gen 		 educ = .
	replace 	 educ = 0 if meta_1_20 == 1
	replace 	 educ = 1 if inlist(meta_1_20,2,3,4)
	replace 	 educ = 2 if meta_1_20 == 5
	replace 	 educ = 3 if inlist(meta_1_20,6,7,8)
	replace 	 educ = 4 if inlist(meta_1_20,9,10,11,12,13,14,15,16)
	replace 	 educ = 5 if inlist(meta_1_20,17,19)
	replace 	 educ = 6 if inlist(meta_1_20,18,20)
	replace 	 educ = 7 if meta_1_20 == 21
	replace 	 educ = 8 if meta_1_20 == 22
	label values educ education
	gen 		 D_tertiary = (inlist(educ,4,5,6,7,8))

	clonevar 	 D_car = intra_1_15_6
	clonevar 	 D_flushtoilet = intra_1_15_7
	clonevar 	 D_computer = intra_1_15_8
	clonevar 	 D_phone = intra_1_15_4
	clonevar 	 D_television = intra_1_15_2
	clonevar 	 D_fridge = intra_1_15_5
	recode 		 D_car D_flushtoilet D_computer D_phone D_television D_fridge (.=0)
	egen 		 hh_assets = rowtotal(intra_1_15_1-intra_1_15_8) // household assets

	
	*** C. Business experience
	clonevar 	 mentors = meta_1_35
	replace 	 mentors = 0 if mentors == .
	clonevar 	 clients = b_bus_1_42
	replace 	 clients = 0 if clients == .

	
	*** D. Joint business ownership
	clonevar 	 own1 = b_bus_1_7a_1
	clonevar 	 own2 = b_bus_1_7a_2
	clonevar 	 own3 = b_bus_1_7a_3
	replace 	 own2 = . if b_bus_1_7a_1 == 1 & b_bus_1_7a_2 == 1
	replace 	 own3 = . if b_bus_1_7a_1 == 1 & b_bus_1_7a_3 == 1
	replace 	 own3 = . if b_bus_1_7a_2 == 1 & b_bus_1_7a_3 == 1
	replace 	 own2 = . if b_bus_1_7a_1 == 2 & b_bus_1_7a_2 == 2
	replace 	 own3 = . if b_bus_1_7a_1 == 2 & b_bus_1_7a_3 == 2
	replace 	 own3 = . if b_bus_1_7a_2 == 2 & b_bus_1_7a_3 == 2
	replace 	 b_bus_1_7 = 1 if own2 == . & own3 == .
	gen 		 D_jointown = ((own1==2|own2==2|own3==2) & (own1==1|own2==1|own3==1))
	replace 	 D_jointown = 1 if intra_1_41_oth == "ክኔጋር አብሮ ይሰራል" // translates to working together

	
	*** E. Spouse employment
	gen 		 D_spousebus = (inlist(intra_1_42,1,2))
	replace 	 D_spousebus = . if meta_1_13 != 2
	gen 		 D_samesector = (intra_1_43 == 1)
	replace 	 D_samesector = . if meta_1_13 != 2

	label define activity_sp 0 "Not working" 1 "Wage employment" 2 "Self-employed", replace
	gen 		 spouse_activity = 0 if inlist(intra_1_41,4,7,8)
	replace 	 spouse_activity = 0 if inlist(intra_1_41_oth,"Na","None")
	replace 	 spouse_activity = 1 if inlist(intra_1_41,1,2)
	replace 	 spouse_activity = 2 if intra_1_41 == 3 | ///
				 inlist(intra_1_41_oth,"ክኔጋር አብሮ ይሰራል","Meselu Small Shop")
	replace 	 spouse_activity = 0 if intra_1_41 == 6
	replace 	 spouse_activity = 0 if spouse_activity == . & intra_1_41 != .
	replace 	 spouse_activity = 2 if D_spousebus == 1
	replace 	 spouse_activity = . if inlist(intra_1_41_oth,"Death","He Is Died", ///
				 "Is not alive anymore","This Hasband Is Died")
	replace 	 spouse_activity = . if meta_1_13 != 2
	label values spouse_activity activity_sp

	label define status 3 "Joint owner/Business in same sector as wife" ///
						2 "Business owner in other sector" 1 "Salaried worker" ///
						0 "Not working", replace
	gen 		 D_sp_status = 0 if inlist(spouse_activity,0)
	replace 	 D_sp_status = 1 if spouse_activity == 1 & D_sp_status == .
	replace 	 D_sp_status = 3 if (D_samesector == 1 | D_jointown == 1) & D_sp_status == .
	replace 	 D_sp_status = 2 if D_spousebus == 1 & D_sp_status == .
	replace 	 D_sp_status = 0 if D_sp_status == .
	replace 	 D_sp_status = . if meta_1_13 != 2
	label values D_sp_status status


	*** F. Gender norms/roles
	foreach i in sp_meta_1_34 sp_meta_1_35 sp_char_1_33 sp_char_1_34 meta_1_new7 {
				 recode `i' (-99 -98 -97 -96 -95 -94 =.)
				 }
	gen 		 D_paychcare = (meta_1_new7 > 0)
	
	foreach i in intra_1_new10 intra_1_new11 intra_1_new12 intra_1_new13 sp_intra_1_41 ///
				 sp_intra_1_42 sp_intra_1_43 {
				 recode `i' (-99 -98 -97 -96 -95 -94 =.)
				 }
 
	* Anderson indices
	swindex 	 D_mainincearn D_hhead D_autonomy, g(IND_roles) normby(control)
	swindex 	 intra_1_new10 intra_1_new11 intra_1_new12 intra_1_new13, g(IND_norms_sjt) ///
				 normby(control) flip(intra_1_new11 intra_1_new13)
				 
	* Standardized Z-scores
	foreach v in intra_1_new10 intra_1_new11 intra_1_new12 intra_1_new13 {
				 qui sum `v'
				 gen `v'_z = ((`v' - r(mean))/r(sd))
				 }

	* Social desirability bias
	foreach i in char_1_new4 char_1_new6 char_1_new7 char_1_new8 {
				 replace `i' = 6 - `i'  if !inlist(`i',0,.) // reverse so that 5 means high SDS
				 }
	egen 		 socbias = rmean(char_1_new4-char_1_new11)
	qui sum 	 socbias if final_status_main == "COMPLETED", d
	gen 		 D_socbias = (socbias > r(p50) & socbias < .)


**#  4. Creating Spouse Outcome Variables
* -----------------------------------------------------------------------------*
	gen 		 D_treated_sp = (sp_meta_1_37 == 1)
	gen 		 HS_complete_sp = (sp_meta_1_38 == 3)
	clonevar 	 D_benefit_sp = sp_meta_1_39
	clonevar 	 nontreated_sp = sp_meta_1_40
	replace 	 nontreated_sp = 1 if sp_meta_1_40 == -96
	label values nontreated_sp meta_1_40_sp

	
	*** A. Demographics
	clonevar 	 age_sp = sp_meta_1_10
	gen 		 educ_sp = .
	replace 	 educ_sp = 0 if sp_meta_1_18 == 1
	replace 	 educ_sp = 1 if inlist(sp_meta_1_18,2,3,4)
	replace 	 educ_sp = 2 if sp_meta_1_18 == 5
	replace 	 educ_sp = 3 if inlist(sp_meta_1_18,6,7,8)
	replace 	 educ_sp = 4 if inlist(sp_meta_1_18,9,10,11,12,13,14,15,16)
	replace 	 educ_sp = 5 if inlist(sp_meta_1_18,17,19)
	replace 	 educ_sp = 6 if inlist(sp_meta_1_18,18,20)
	replace 	 educ_sp = 7 if sp_meta_1_18 == 21
	replace 	 educ_sp = 8 if sp_meta_1_18 == 22
	label values educ_sp education
	gen 		 D_tertiary_sp = (inlist(educ_sp,4,5,6,7,8))

	
	*** B. Business activities
	clonevar 	 experience_sp = sp_h_labor_1_5
	replace 	 experience_sp = 0 if experience_sp == .
	gen 		 D_paychcare_sp = (sp_meta_1_29 > 0)
	gen 		 D_opportunity_sp = (sp_h_labor_1_14_1==1|sp_h_labor_1_14_2==1|sp_h_labor_1_14_3==1)

	label define activity_sp2 0 "Unemployed" 1 "Wage employment" 2 "Business owner" ///
							  3 "Agricultural activity", replace
	clonevar 	 activity_sp = sp_h_labor_1_2
	replace 	 activity_sp = "0" if sp_h_labor_1_2 == "4"
	replace 	 activity_sp = "2" if sp_h_labor_1_2 == "1 2" // priority to business owner
	destring 	 activity_sp, replace
	label values activity_sp activity_sp2

	* Convert to real values
	gen double 	 date2 = date(sp_today, "DMY")
	format 		 date2 %td
	gen 		 month2 = month(date2)

	foreach i in sp_h_labor_1_16_1 sp_h_labor_1_17_1 sp_h_labor_1_20_1 sp_meta_1_20 {
				 replace `i' = `i'/(3.376) if month2 == 6
				 replace `i' = `i'/(3.448) if month2 == 7
				 replace `i' = `i'/(3.521) if month2 == 8
				 replace `i' = `i'/(3.596) if month2 == 9
				 }

	* Winsorinzing
	foreach i in sp_h_labor_1_16_1 sp_h_labor_1_17_1 sp_h_labor_1_18_1 ///
				 sp_h_labor_1_20_1 sp_meta_1_20 {
				 winsor `i', gen (win99_`i') p(0.01)	
				 winsor `i', gen (win95_`i') p(0.05)
				 winsor `i', gen (win90_`i') p(0.10)
				 winsor `i', gen (win85_`i') p(0.15)
				 winsor `i', gen (win80_`i') p(0.20)
				 winsor `i', gen (win75_`i') p(0.25)
				 gen ihs_`i' = asinh(`i')
				 gen log_`i' = log(`i')
				 }


	*** C. Socio-emotional skills
	foreach i in sp_char_1_50 sp_char_1_52 sp_char_1_55 sp_char_1_56 sp_char_1_57 {
				 replace `i' = 6 - `i' if !inlist(`i',0,.) // reverse values so that 5 means high
				 }
	egen 		 selfesteem_sp = rmean(sp_char_1_48-sp_char_1_57)
	egen 		 selfefficacy_sp = rmean(sp_char_1_2-sp_char_1_11)
	egen 		 proactivity_sp = rmean(sp_char_1_16-sp_char_1_23)
	egen 		 express_sp = rmean(sp_char_1_43-sp_char_1_47)

	* Stardardized z-scores
	foreach v in selfesteem_sp selfefficacy_sp proactivity_sp express_sp {
				 qui sum `v'
				 gen `v'_z = ((`v' - r(mean))/r(sd))
				 }

	* Anderson index
	swindex 	 sp_char_1_48-sp_char_1_57 sp_char_1_2-sp_char_1_11 ///
				 sp_char_1_16-sp_char_1_23 sp_char_1_43-sp_char_1_47, ///
				 g(IND_ses_sp) normby(control)


	*** D. Marriage quality
	egen 		 coop_sp = rowtotal(sp_intra_1_15_1 sp_intra_1_15_2 sp_intra_1_15_3 sp_intra_1_15_4), missing
	egen 		 empathy_sp = rowmean(sp_char_1_24-sp_char_1_28)
	egen 		 commun_sp = rowmean(sp_char_1_30-sp_char_1_31)
	replace 	 sp_char_1_15 = 6 - sp_char_1_15
	egen 		 trust_sp = rmean(sp_char_1_13 sp_char_1_14 sp_char_1_15)

	clonevar 	 intimacy_sp = sp_intra_1_36
	replace 	 intimacy_sp = 6 - intimacy_sp if !inlist(sp_intra_1_36,0,.)
	label values intimacy_sp intimate
	clonevar 	 qualmarriage_sp = sp_char_1_12
	clonevar 	 quallife_sp = sp_char_1_32
	replace 	 quallife_sp = 5 - quallife if !inlist(sp_char_1_32,0,.)
	label values quallife_sp satisfied

	* Stardardized z-scores
	foreach v in coop_sp empathy_sp commun_sp trust_sp intimacy_sp ///
				 qualmarriage_sp quallife_sp {
				 qui sum `v'
				 gen `v'_z = ((`v' - r(mean))/r(sd))
				 }

	* Anderson index
	swindex 	 sp_intra_1_15_1 sp_intra_1_15_2 sp_intra_1_15_3 sp_intra_1_15_4 ///
				 sp_char_1_24-sp_char_1_28 sp_char_1_30-sp_char_1_31 sp_char_1_13 ///
				 sp_char_1_14 sp_char_1_15, g(IND_marriage_sp) normby(control)


	*** E. Decision-making
	foreach i of numlist 3 4 5 7 {
		gen 	 sp_decision`i' = . if inlist(sp_intra_1_`i'a,""," ","-97")
		replace  sp_decision`i' = 1 if sp_intra_1_`i'a == "2"
		replace  sp_decision`i' = 2 if sp_intra_1_`i'a == "1"
		replace  sp_decision`i' = 3 if (sp_intra_1_`i'a == "1 2" | sp_intra_1_`i'a == "1 2 3"| /// 
				 sp_intra_1_`i'a == "2 3")
		replace  sp_decision`i' = 4 if (sp_intra_1_`i'a == "3"|sp_intra_1_`i'a == "1 3")
		replace  sp_decision`i' = 5 if (sp_intra_1_`i'a == "1 2 4")
		lab val  sp_decision`i' decision_making
	 
		gen 	 D_self_decision`i'_sp = (inlist(sp_decision`i',1))
		replace  D_self_decision`i'_sp = . if sp_decision`i' == .
		gen 	 D_husb_decision`i'_sp = (inlist(sp_decision`i',2))
		replace  D_husb_decision`i'_sp = . if sp_decision`i' == .
		gen 	 D_joint_decision`i'_sp = (inlist(sp_decision`i',3))
		replace  D_joint_decision`i'_sp = . if sp_decision`i' == .
		}

	* Anderson indices
	swindex 	 D_self_decision3_sp D_self_decision4_sp D_self_decision5_sp, ///
				 g(IND_self_hh_decision_sp) normby(control)
	swindex 	 D_husb_decision3_sp D_husb_decision4_sp D_husb_decision5_sp, ///
				 g(IND_husb_hh_decision_sp) normby(control)
	swindex 	 D_joint_decision3_sp D_joint_decision4_sp D_joint_decision5_sp, ///
				 g(IND_joint_hh_decision_sp) normby(control)
			  
	* Simple indices
	egen 		 D_self_hh_decision_sp = anymatch(D_self_decision3_sp D_self_decision4_sp ///
				 D_self_decision5_sp), v(1)
	egen 		 D_husb_hh_decision_sp = anymatch(D_husb_decision3_sp D_husb_decision4_sp ///
				 D_husb_decision5_sp), v(1)							 
	egen 		 D_joint_hh_decision_sp = anymatch(D_joint_decision3_sp D_joint_decision4_sp ///
				 D_joint_decision5_sp), v(1)							 
				
				
	*** F. Intra-household resource sharing							  
	foreach i in sp_meta_1_30 sp_meta_1_31 sp_meta_1_32 sp_meta_1_33 {
		recode `i' (-99 -98 -97 -96 -95 -94 =0)
		}

	clonevar 	 time_work_resp_sp     = sp_meta_1_32
	replace 	 time_work_resp_sp     = . if meta_1_13 != 2
	clonevar 	 time_work_spouse_sp   = sp_meta_1_30
	replace 	 time_work_spouse_sp   = . if meta_1_13 != 2
	clonevar 	 time_chores_resp_sp   = sp_meta_1_33
	replace 	 time_chores_resp_sp   = . if meta_1_13 != 2
	clonevar 	 time_chores_spouse_sp = sp_meta_1_31
	replace 	 time_chores_spouse_sp = . if meta_1_13 != 2

	egen 		 time_sp = rowtotal(time_chores_resp_sp time_chores_spouse_sp), missing
	gen 		 sp_share_chores = (time_chores_resp_sp/time_sp)*100
	replace 	 sp_share_chores = 0 if time_sp == 0
	gen 		 sp_share_chores_sp = (time_chores_spouse_sp/time_sp)*100
	replace 	 sp_share_chores_sp = 0 if time_sp == 0							  
							  						  
	gen 		 D_assetjoint_sp = (sp_intra_1_9 == 3)
	gen 		 D_direct_help_sp = (sp_intra_1_17 == 1 | sp_intra_1_18 == 1)

	gen 		 D_foodprep_sp = (!inlist(sp_intra_1_14a,7,8,9))
	gen 		 D_clean_sp = (!inlist(sp_intra_1_14b,7,8,9))
	gen 		 D_chcare_sp = (!inlist(sp_intra_1_14c,7,8,9))
	replace 	 D_foodprep_sp = . if meta_1_13 != 2
	replace 	 D_clean_sp    = . if meta_1_13 != 2
	replace 	 D_chcare_sp   = . if meta_1_13 != 2
	gen 		 D_support_house_sp = (!inlist(sp_intra_1_14a,7,8,9) | ///
				 !inlist(sp_intra_1_14b,7,8,9) | !inlist(sp_intra_1_14c,7,8,9))

	gen 		 D_knowledge_sp = (sp_intra_1_27 == 1)
	gen 		 D_networks_sp = (sp_intra_1_28 == 1)
	egen 		 support_emotion_sp = rmean(sp_intra_1_37-sp_intra_1_40)
	replace 	 support_emotion_sp = . if meta_1_13 != 2
	qui sum 	 support_emotion_sp
	gen 		 support_emotion_sp_z = ((support_emotion_sp - r(mean))/r(sd))
	gen 		 D_finance_sp = (sp_intra_1_22 == 1)

	* Overall impact
	egen 		 D_support_any_sp = anymatch(D_direct_help_sp D_knowledge_sp ///
				 D_networks_sp D_finance_sp), v(1)
				 
	egen 		 benefit_spouse_sp = rmean(sp_intra_1_33a-sp_intra_1_33f)
	replace 	 benefit_spouse_sp = . if meta_1_13 != 2
	qui sum 	 benefit_spouse_sp
	gen 		 benefit_spouse_sp_z = ((benefit_spouse_sp - r(mean))/r(sd))
				 
	swindex 	 D_direct_help_sp D_support_house_sp D_knowledge_sp D_networks_sp D_finance_sp ///
				 sp_intra_1_37-sp_intra_1_40, g(IND_support_sp) normby(control)
	replace 	 IND_support_sp = . if meta_1_13 != 2


	*** G. Gender norms
	foreach i in sp_meta_1_34 sp_meta_1_35 sp_intra_1_41 sp_intra_1_42 sp_intra_1_43 ///
				 sp_char_1_33 sp_char_1_34 {
				 recode `i' (-99 -98 -97 -96 -95 -94 =0)
				 }

	swindex 	 sp_char_1_33 sp_meta_1_34, g(IND_beliefs_sp) normby(control) ///
				 flip(sp_char_1_33 sp_meta_1_34)
	swindex 	 sp_char_1_34 sp_meta_1_35, g(IND_norms_sp) normby(control) ///
				 flip(sp_char_1_34 sp_meta_1_35)
	swindex 	 sp_intra_1_41 sp_intra_1_42 sp_intra_1_43, g(IND_norms_sjt_sp) ///
				 normby(control) flip(sp_intra_1_42)

	* Social desirability bias
	foreach i in sp_char_1_35 sp_char_1_37 sp_char_1_38 sp_char_1_39 {
				 replace `i' = 6 - `i'  if !inlist(`i',0,.)
				 }
	egen 		 socbias_sp = rmean(sp_char_1_35-sp_char_1_42)
	qui sum 	 socbias_sp if final_status_spouse == "COMPLETED", d
	gen 		 D_socbias_sp = (socbias_sp > r(p50) & socbias_sp < .)


**#  5. Intermediate Dataset 
* -----------------------------------------------------------------------------*	
	qui ds		 meta_1_8-main_business_sector machinery-D_socbias
	foreach v of varlist `r(varlist)' {
				 cap replace `v' = .  if final_status_main != "COMPLETED"
				 cap replace `v' = "" if final_status_main != "COMPLETED"
				 }
	qui ds 		 sp_meta_1_8-sp_fin_1_16_5 D_treated_sp-D_socbias_sp
	foreach v of varlist `r(varlist)' {
				 cap replace `v' = .  if final_status_spouse != "COMPLETED"
				 cap replace `v' = "" if final_status_spouse != "COMPLETED"
				 }
	
	foreach v of varlist _all {
				 label variable `v' "" // survey labels are too long for Stata12 
				 }
	
	rename 		 (meta_1_1 resp_id) (resp_id meta_1_1)
	saveold		 "$data_working/DOT_Endline_data.dta", replace version(12.1)

	
********************************************************************************
*                        END OF CONSTRUCTION DOFILE                            *
********************************************************************************
	